/**
 * Any CSS included here will be global. The classic template
 * bundles Infima by default. Infima is a CSS framework designed to
 * work well for content-centric websites.
 */

/* You can override the default Infima variables here. */
:root {
  --ifm-color-primary: #cccccc;
  --ifm-color-primary-dark: #b3b3b3;
  --ifm-color-primary-darker: #a6a6a6;
  --ifm-color-primary-darkest: #8c8c8c;
  --ifm-color-primary-light: #d9d9d9;
  --ifm-color-primary-lighter: #e6e6e6;
  --ifm-color-primary-lightest: #f2f2f2;
  --ifm-code-font-size: 95%;
  --docusaurus-highlighted-code-line-bg: rgba(0, 0, 0, 0.1);
}

/* For readability concerns, you should choose a lighter palette in dark mode. */
[data-theme='dark'] {
  --ifm-color-primary: #595959;
  --ifm-color-primary-dark: #4d4d4d;
  --ifm-color-primary-darker: #404040;
  --ifm-color-primary-darkest: #333333;
  --ifm-color-primary-light: #666666;
  --ifm-color-primary-lighter: #737373;
  --ifm-color-primary-lightest: #808080;
  --docusaurus-highlighted-code-line-bg: rgba(0, 0, 0, 0.3);
}

.hero__subtitle {
  color: #2f4476;
}

.button--secondary {
  color: #2f4476;
  border-color: #2f4476;
}